local mvec3_set = mvector3.set
local mvec3_set_z = mvector3.set_z
local mvec3_sub = mvector3.subtract
local mvec3_norm = mvector3.normalize
local mvec3_dir = mvector3.direction
local mvec3_set_l = mvector3.set_length
local mvec3_add = mvector3.add
local mvec3_dot = mvector3.dot
local mvec3_cross = mvector3.cross
local mvec3_rot = mvector3.rotate_with
local mvec3_rand_orth = mvector3.random_orthogonal
local mvec3_lerp = mvector3.lerp
local mrot_axis_angle = mrotation.set_axis_angle
local temp_vec1 = Vector3()
local temp_vec2 = Vector3()
local temp_vec3 = Vector3()
local temp_rot1 = Rotation()
local bezier_curve = {
	0,
	0,
	1,
	1
}
CopActionShoot = CopActionShoot or class()
CopActionShoot._ik_presets = {
	spine = {
		update = "_update_ik_spine",
		start = "_begin_ik_spine",
		get_blend = "_get_blend_ik_spine",
		stop = "_stop_ik_spine"
	},
	r_arm = {
		update = "_update_ik_r_arm",
		start = "_begin_ik_r_arm",
		get_blend = "_get_blend_ik_r_arm",
		stop = "_stop_ik_r_arm"
	}
}

function CopActionShoot:init(action_desc, common_data)
	self._common_data = common_data	
	local t = TimerManager:game():time()
	self._ext_movement = common_data.ext_movement
	self._ext_anim = common_data.ext_anim
	self._ext_brain = common_data.ext_brain
	self._ext_inventory = common_data.ext_inventory
	self._ext_base = common_data.ext_base
	self._body_part = action_desc.body_part
	self._machine = common_data.machine
	self._unit = common_data.unit
	local preset_name = self._ext_anim.base_aim_ik or "spine"
	local preset_data = self._ik_presets[preset_name]
	self._ik_preset = preset_data

	self[preset_data.start](self)

	local weapon_unit = self._ext_inventory:equipped_unit()
	local weap_tweak = weapon_unit:base():weapon_tweak_data()
	local weapon_usage_tweak = common_data.char_tweak.weapon[weap_tweak.usage]

	if not weapon_usage_tweak then
		Application:error("[CopActionShoot:init vent] unknown weapon used: ", inspect(self._unit), "   ", inspect(weapon_unit), "   ", inspect(weap_tweak), "   ", inspect(common_data.char_tweak))

		return
	end

	self._weapon_unit = weapon_unit
	self._weapon_base = weapon_unit:base()
	self._weap_tweak = weap_tweak
	self._w_usage_tweak = weapon_usage_tweak
	self._reload_speed = weapon_usage_tweak.RELOAD_SPEED
	self._spread = weapon_usage_tweak.spread
	self._falloff = weapon_usage_tweak.FALLOFF
	self._variant = action_desc.variant
	self._body_part = action_desc.body_part
	self._turn_allowed = Network:is_client()
	self._automatic_weap = weap_tweak.auto and true
	self._shoot_t = 0
	self._melee_timeout_t = t + 1
	local shoot_from_pos = self._ext_movement:m_head_pos()
	self._shoot_from_pos = shoot_from_pos

	self:on_attention(common_data.attention)

	if Network:is_server() then
		self._ext_movement:set_stance_by_code(3)
	end
	CopActionAct._create_blocks_table(self, action_desc.blocks)

	if Network:is_server() then
		common_data.ext_network:send("action_aim_state", true)
	end

	self._skipped_frames = 1
	self._get_target_position = self._get_target_pos
	
	self._commander_effect = nil
	self._commander_buff_time = nil
	self._converted_chk = self._unit:brain().is_converted_chk and self._unit:brain():is_converted_chk() and true or nil
	
	return true
end

function CopActionShoot:on_exit()
	self._commander_buff_time = nil

	if self._commander_effect then
		World:effect_manager():fade_kill(self._commander_effect)
		self._commander_effect = nil
	end	

	if Network:is_server() then
		self._ext_movement:set_stance("hos")
	end

	if self._modifier_on then
		self[self._ik_preset.stop](self)
	end

	if self._autofiring then
		self._weapon_base:stop_autofire()
	end

	if self._ext_anim.recoil then
		self._ext_movement:play_redirect("up_idle")
	end

	if Network:is_server() then
		self._common_data.unit:network():send("action_aim_state", false)
	end

	if self._shooting_player and alive(self._attention.unit) then
		self._attention.unit:movement():on_targetted_for_attack(false, self._common_data.unit)
	end
	
end

function CopActionShoot:update(t)
	local vis_state = self._ext_base:lod_stage()
	vis_state = vis_state or 4

	if vis_state == 1 then
		-- Nothing
	elseif self._skipped_frames < vis_state * 3 then
		self._skipped_frames = self._skipped_frames + 1

		return
	else
		self._skipped_frames = 1
	end

	local shoot_from_pos = self._shoot_from_pos
	local ext_anim = self._ext_anim
	local target_vec, target_dis, autotarget, target_pos = nil

	if self._attention then
		target_pos, target_vec, target_dis, autotarget = self:_get_target_position(shoot_from_pos, self._attention, t)
		local tar_vec_flat = temp_vec2

		mvec3_set(tar_vec_flat, target_vec)
		mvec3_set_z(tar_vec_flat, 0)
		mvec3_norm(tar_vec_flat)

		local fwd = self._common_data.fwd
		local fwd_dot = mvec3_dot(fwd, tar_vec_flat)
		--COMMANDER BUFF SHIT AW YEAH
		if not self._converted_chk and self._unit:in_slot(12) then
			if not self._commander_buff_time or self._commander_buff_time < t then
			
				if self._commander_buff_time and self._commander_buff_time < t then
					if self._commander_effect then
						World:effect_manager():fade_kill(self._commander_effect)
						self._commander_effect = nil
					end	
					self._commander_buff_time = nil
				end
				
				local enemies = World:find_units_quick(self._unit, "sphere", self._unit:position(), 1200, managers.slot:get_mask("enemies"))
				
				if enemies then
					for _, enemy in ipairs(enemies) do
						if enemy:base()._tweak_table == "german_commander" or enemy:base()._tweak_table == "german_og_commander" or enemy:base()._tweak_table == "german_officer" then
							local tase_effect_table = self._unit:character_damage() ~= nil and self._unit:character_damage()._tase_effect_table
								
							if tase_effect_table and not self._commander_effect then
								self._commander_effect = World:effect_manager():spawn(tase_effect_table)	
							end
							self._commander_buff_time = t + 2
						end
					end
				end
			end
		end
		
		if self._turn_allowed then
			local active_actions = self._common_data.active_actions
			local queued_actions = self._common_data.queued_actions

			if (not active_actions[2] or active_actions[2]:type() == "idle") and (not queued_actions or not queued_actions[1] and not queued_actions[2]) and not self._ext_movement:chk_action_forbidden("walk") then
				local fwd_dot_flat = mvec3_dot(tar_vec_flat, fwd)

				if fwd_dot_flat < 0.87 then
					local spin = tar_vec_flat:to_polar_with_reference(fwd, math.UP).spin
					local new_action_data = {
						body_part = 2,
						type = "turn",
						angle = spin
					}

					self._ext_movement:action_request(new_action_data)
				end
			end
		end

		target_vec = self:_upd_ik(target_vec, fwd_dot, t)
	else
		if self._commander_effect then
			World:effect_manager():fade_kill(self._commander_effect)
			self._commander_effect = nil
		end	
		
		self._commander_buff_time = nil
	end
		

	if not ext_anim.reload and not ext_anim.equip and not ext_anim.melee then
		if ext_anim.equip then
			-- Nothing
		elseif self._weapon_base:clip_empty() then
			if self._autofiring then
				self._weapon_base:stop_autofire()

				if ext_anim.recoil then
					self._ext_movement:play_redirect("up_idle")
				end

				self._autofiring = nil
				self._autoshots_fired = nil
			end

			if not ext_anim.recoil then
				if self._ext_anim.base_no_reload then
					self._weapon_unit:base():on_reload()
				else
					local res = CopActionReload._play_reload(self)

					if res then
						self._machine:set_speed(res, self._reload_speed)
					end
				end
			end
		elseif self._autofiring then
			if not target_vec or not self._common_data.allow_fire then
				self._weapon_base:stop_autofire()

				self._shoot_t = t + 0.6
				self._autofiring = nil
				self._autoshots_fired = nil

				if ext_anim.recoil then
					self._ext_movement:play_redirect("up_idle")
				end
			else
				local spread = self._spread
				local falloff, i_range = self:_get_shoot_falloff(target_dis, self._falloff)
				local new_target_pos = self._shoot_history and self:_get_unit_shoot_pos(t, target_pos, target_dis, self._w_usage_tweak, falloff, i_range, autotarget)

				if new_target_pos then
					target_pos = new_target_pos
				else
					spread = math.min(20, spread)
				end

				local spread_pos = temp_vec2

				mvec3_rand_orth(spread_pos, target_vec)
				mvec3_set_l(spread_pos, spread)
				mvec3_add(spread_pos, target_pos)

				target_dis = mvec3_dir(target_vec, shoot_from_pos, spread_pos)
				local damage_multiplier = self:_get_shoot_falloff_damage(self._falloff, target_dis, i_range)
				local fired = self._weapon_base:trigger_held(shoot_from_pos, target_vec, damage_multiplier, self._shooting_player, nil, nil, nil, self._attention.unit)
				-- non husk commander effect
				if self._commander_effect then
					damage_multiplier = damage_multiplier + 1
				end
				
				if fired then
					if fired.hit_enemy and fired.hit_enemy.type == "death" and self._unit:unit_data().mission_element then
						self._unit:unit_data().mission_element:event("killshot", self._unit)
					end

					if not ext_anim.recoil and vis_state == 1 and not ext_anim.base_no_recoil then
						self._ext_movement:play_redirect("recoil_auto")
					end

					if not self._autofiring or self._autoshots_fired == self._autofiring - 1 then
						self._autofiring = nil
						self._autoshots_fired = nil

						self._weapon_base:stop_autofire()

						if ext_anim.recoil then
							self._ext_movement:play_redirect("up_idle")
						end

						if vis_state == 1 then
							self._shoot_t = t + (self._common_data.is_suppressed and 1.5 or 1) * math.lerp(falloff.recoil[1], falloff.recoil[2], math.random())
						else
							self._shoot_t = t + falloff.recoil[2]
						end
					else
						self._autoshots_fired = self._autoshots_fired + 1
					end
				end
			end
		elseif target_vec and self._common_data.allow_fire and self._shoot_t < t and self._mod_enable_t < t then
			local shoot = nil

			if autotarget or self._shooting_husk_player and self._next_vis_ray_t < t then
				if self._shooting_husk_player then
					self._next_vis_ray_t = t + 2
				end

				local fire_line = World:raycast("ray", shoot_from_pos, target_pos, "slot_mask", self._verif_slotmask, "ray_type", "ai_vision")

				if fire_line then
					if t - self._line_of_sight_t > 3 then
						local aim_delay_minmax = self._w_usage_tweak.aim_delay
						local lerp_dis = math.min(1, target_vec:length() / self._falloff[#self._falloff].r)
						local aim_delay = math.lerp(aim_delay_minmax[1], aim_delay_minmax[2], lerp_dis)
						aim_delay = aim_delay + math.random() * aim_delay * 0.3

						if self._common_data.is_suppressed then
							aim_delay = aim_delay * 1.5
						end

						self._shoot_t = t + aim_delay
					elseif fire_line.distance > 300 then
						shoot = true
					end
				else
					if t - self._line_of_sight_t > 1 and not self._last_vis_check_status then
						local shoot_hist = self._shoot_history
						local displacement = mvector3.distance(target_pos, shoot_hist.m_last_pos)
						local focus_delay = self._w_usage_tweak.focus_delay * math.min(1, displacement / self._w_usage_tweak.focus_dis)
						shoot_hist.focus_start_t = t
						shoot_hist.focus_delay = focus_delay
						shoot_hist.m_last_pos = mvector3.copy(target_pos)
					end

					self._line_of_sight_t = t
					shoot = true
				end

				self._last_vis_check_status = shoot
			elseif self._shooting_husk_player then
				shoot = self._last_vis_check_status
			else
				shoot = true
			end

			if shoot then
				local melee = nil

				if autotarget and (not self._common_data.melee_countered_t or t - self._common_data.melee_countered_t > 15) and target_dis < 130 and self._w_usage_tweak.melee_speed and self._melee_timeout_t < t then
					melee = self:_chk_start_melee(target_vec, target_dis, autotarget, target_pos)
				end

				if not melee then
					local falloff, i_range = self:_get_shoot_falloff(target_dis, self._falloff)
					local firemode = nil

					if self._automatic_weap then
						local random_mode = math.random()

						for i_mode, mode_chance in ipairs(falloff.mode) do
							if random_mode <= mode_chance then
								firemode = i_mode

								break
							end
						end
					else
						firemode = 1
					end

					if firemode > 1 then
						self._weapon_base:start_autofire(firemode < 4 and firemode)

						self._autofiring = firemode < 4 and firemode or math.random(self._w_usage_tweak.autofire_rounds[1], self._w_usage_tweak.autofire_rounds[2])
						self._autoshots_fired = 0

						if vis_state == 1 and not ext_anim.base_no_recoil then
							self._ext_movement:play_redirect("recoil_auto")
						end
					else
						local spread = self._spread

						if autotarget then
							local new_target_pos = self._shoot_history and self:_get_unit_shoot_pos(t, target_pos, target_dis, self._w_usage_tweak, falloff, i_range, autotarget)

							if new_target_pos then
								target_pos = new_target_pos
							else
								spread = math.min(20, spread)
							end
						end

						local spread_pos = temp_vec2

						mvec3_rand_orth(spread_pos, target_vec)
						mvec3_set_l(spread_pos, spread)
						mvec3_add(spread_pos, target_pos)

						target_dis = mvec3_dir(target_vec, shoot_from_pos, spread_pos)
						
						local damage_multiplier = self:_get_shoot_falloff_damage(self._falloff, target_dis, i_range)
						
						local fired = self._weapon_base:singleshot(shoot_from_pos, target_vec, damage_multiplier, self._shooting_player, nil, nil, nil, self._attention.unit)
						-- husk commander effect
						if self._commander_effect then
							damage_multiplier = damage_multiplier + 1
						end
	
						if fired and fired.hit_enemy and fired.hit_enemy.type == "death" and self._unit:unit_data().mission_element then
							self._unit:unit_data().mission_element:event("killshot", self._unit)
						end

						if vis_state == 1 then
							if not ext_anim.base_no_recoil then
								self._ext_movement:play_redirect("recoil_single")
							end

							self._shoot_t = t + (self._common_data.is_suppressed and 1.5 or 1) * math.lerp(falloff.recoil[1], falloff.recoil[2], math.random())
						else
							self._shoot_t = t + falloff.recoil[2]
						end
					end
				end
			end
		end
	end

	if self._ext_anim.base_need_upd then
		self._ext_movement:upd_m_head_pos()
	end
end