CopDamage = CopDamage or class()
CopDamage._all_event_types = {
	"dmg_rcv",
	"light_hurt",
	"hurt",
	"heavy_hurt",
	"death",
	"shield_knock",
	"stun",
	"counter_tased",
	"taser_tased"
}
CopDamage._ATTACK_VARIANTS = {
	"explosion",
	"stun",
	"fire"
}
CopDamage._HEALTH_GRANULARITY = 512
CopDamage.WEAPON_TYPE_GRANADE = 1
CopDamage.WEAPON_TYPE_BULLET = 2
CopDamage.WEAPON_TYPE_FLAMER = 3
CopDamage.DEBUG_HP = CopDamage.DEBUG_HP or false
CopDamage._hurt_severities = {
	heavy = "heavy_hurt",
	fire = "fire_hurt",
	poison = "poison_hurt",
	explode = "expl_hurt",
	light = "light_hurt",
	moderate = "hurt",
	none = false
}
CopDamage._impact_bones = {}
local impact_bones_tmp = {
	"Hips",
	"Spine",
	"Spine1",
	"Spine2",
	"Neck",
	"Head",
	"LeftShoulder",
	"LeftArm",
	"LeftForeArm",
	"RightShoulder",
	"RightArm",
	"RightForeArm",
	"LeftUpLeg",
	"LeftLeg",
	"LeftFoot",
	"RightUpLeg",
	"RightLeg",
	"RightFoot"
}

for i, k in ipairs(impact_bones_tmp) do
	local name_ids = Idstring(impact_bones_tmp[i])
	CopDamage._impact_bones[name_ids:key()] = name_ids
end

impact_bones_tmp = nil
local mvec_1 = Vector3()
local mvec_2 = Vector3()

function CopDamage:init(unit)
	self._unit = unit
	local char_tweak = tweak_data.character[unit:base()._tweak_table]
	self._HEALTH_INIT = char_tweak.HEALTH_INIT

	if managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_ENEMY_HEALTH) then
		self._HEALTH_INIT = self._HEALTH_INIT * (managers.buff_effect:get_effect_value(BuffEffectManager.EFFECT_ENEMY_HEALTH) or 1)
	end

	self._health = self._HEALTH_INIT
	self._health_ratio = 1
	self._HEALTH_INIT_PRECENT = self._HEALTH_INIT / self._HEALTH_GRANULARITY
	self._autotarget_data = {
		fast = unit:get_object(Idstring("Spine1"))
	}
	self._pickup = nil
	self._listener_holder = EventListenerHolder:new()

	if char_tweak.permanently_invulnerable or self.immortal then
		self:set_invulnerable(true)
	end

	self._char_tweak = char_tweak
	self._spine2_obj = unit:get_object(Idstring("Spine2"))

	if self._head_body_name then
		self._ids_head_body_name = Idstring(self._head_body_name)
		self._head_body_key = self._unit:body(self._head_body_name):key()
	end

	self._ids_plate_name = Idstring("body_plate")
	self._has_plate = true
	self._last_time_unit_got_fire_damage = nil
	self._last_time_unit_got_fire_effect = nil
	self._temp_flame_redir_res = nil
	self._active_fire_bone_effects = {}

	if CopDamage.DEBUG_HP then
		self:_create_debug_ws()
	end

	self._tase_effect_table = {
		effect = Idstring("effects/vanilla/character/taser_hittarget_001"),
		parent = self._spine2_obj
	}
	
	self._last_damage_direction = nil
	self._dismembered_parts = {}
	
end

function CopDamage:damage_bullet(attack_data)
	self._last_damage_direction = attack_data.col_ray.ray

	if attack_data.damage < 0.001 then
		return "no_damage"
	elseif self._dead or self._invulnerable then
		return
	end

	local death_event_params = {
		damage_type = WeaponTweakData.DAMAGE_TYPE_BULLET,
		enemy_type = tweak_data.character[self._unit:base()._tweak_table].type,
		special_enemy_type = tweak_data.character[self._unit:base()._tweak_table].special_type
	}

	if self._has_plate and attack_data.col_ray.body and attack_data.col_ray.body:name() == self._ids_plate_name then
		local armor_pierce_roll = math.rand(1)
		local armor_pierce_value = 0

		if attack_data.attacker_unit == managers.player:player_unit() and not attack_data.weapon_unit:base().thrower_unit then
			armor_pierce_value = armor_pierce_value + attack_data.weapon_unit:base():armor_piercing_chance()
			armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("player", "armor_piercing_chance", 0)
			armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("weapon", "armor_piercing_chance", 0)
			armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("weapon", "armor_piercing_chance_2", 0)

			if attack_data.weapon_unit:base():got_silencer() then
				armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("weapon", "armor_piercing_chance_silencer", 0)
			end

			local weapon_category = attack_data.weapon_unit:base():weapon_tweak_data().category

			if weapon_category == "saw" then
				armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("saw", "armor_piercing_chance", 0)
			end
		elseif attack_data.attacker_unit:base() and attack_data.attacker_unit:base().sentry_gun then
			local owner = attack_data.attacker_unit:base():get_owner()

			if alive(owner) then
				if owner == managers.player:player_unit() then
					armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("sentry_gun", "armor_piercing_chance", 0)
					armor_pierce_value = armor_pierce_value + managers.player:upgrade_value("sentry_gun", "armor_piercing_chance_2", 0)
				else
					armor_pierce_value = armor_pierce_value + (owner:base():upgrade_value("sentry_gun", "armor_piercing_chance") or 0)
					armor_pierce_value = armor_pierce_value + (owner:base():upgrade_value("sentry_gun", "armor_piercing_chance_2") or 0)
				end
			end
		end

		if armor_pierce_roll >= armor_pierce_value then
			return
		end
	end

	local result = nil
	local body_index = self._unit:get_body_index(attack_data.col_ray.body:name())
	local head = self._head_body_name and attack_data.col_ray.body and attack_data.col_ray.body:name() == self._ids_head_body_name
	local instakill = false
	local damage = attack_data.damage

	if PlayerDamage.is_friendly_fire(self, attack_data.attacker_unit) then
		damage = damage * 24
	end
	
	if self._unit:base():char_tweak().DAMAGE_CLAMP_BULLET then
		damage = math.min(damage, self._unit:base():char_tweak().DAMAGE_CLAMP_BULLET)
	end

	damage = damage * (self._marked_dmg_mul or 1)

	if self._unit:movement():cool() then
		damage = self._HEALTH_INIT
	end

	damage = damage * managers.player:team_upgrade_value("player", "warcry_damage_multiplier", 1) * managers.player:team_upgrade_value("player", "warcry_damage_buff_bonus", 1)
	local headshot_multiplier = 1
	local weapon_headshot_multiplier = 1

	if attack_data.attacker_unit == managers.player:player_unit() then
		local percent_dmg = damage / self._HEALTH_INIT
		local critical_hit, crit_damage = self:roll_critical_hit(damage)

		if critical_hit then
			death_event_params.critical_hit = true
			damage = crit_damage
		elseif head then
			death_event_params.headshot = true
		end

		headshot_multiplier = headshot_multiplier * managers.player:upgrade_value("player", "headshot_damage_multiplier", 1)

		if managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_PLAYER_HEADSHOT_DAMAGE) and head then
			headshot_multiplier = headshot_multiplier * (managers.buff_effect:get_effect_value(BuffEffectManager.EFFECT_PLAYER_HEADSHOT_DAMAGE) or 1)
		end

		if tweak_data.character[self._unit:base()._tweak_table].priority_shout then
			damage = damage * managers.player:upgrade_value("weapon", "special_damage_taken_multiplier", 1)
		end

		if alive(attack_data.weapon_unit) and attack_data.weapon_unit:base().weapon_tweak_data then
			weapon_headshot_multiplier = attack_data.weapon_unit:base():weapon_tweak_data().headshot_multiplier or 1
		end
	end

	if self._damage_reduction_multiplier then
		damage = damage * self._damage_reduction_multiplier
	elseif head then
		if self._char_tweak.headshot_dmg_mul then
			damage = damage * weapon_headshot_multiplier * self._char_tweak.headshot_dmg_mul * headshot_multiplier
		else
			instakill = true
			damage = self._health * 10
		end
	end

	damage = self:_apply_damage_modifier(damage, attack_data)

	if self._unit:unit_data().turret_weapon then
		death_event_params.mounted_on_turret = true
		local turret_tweak_data = tweak_data.weapon[self._unit:unit_data().turret_weapon._name_id]
		local damage_multiplier = turret_tweak_data.puppet_damage_multiplier or 1
		damage = damage * damage_multiplier
	end

	local damage_percent = math.ceil(math.clamp(damage / self._HEALTH_INIT_PRECENT, 1, self._HEALTH_GRANULARITY))
	damage = damage_percent * self._HEALTH_INIT_PRECENT
	damage, damage_percent = self:_apply_min_health_limit(damage, damage_percent)

	if self._immortal then
		damage = 0
		damage_percent = 0
	end

	if managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_ENEMIES_DIE_ONLY_ON_HEADSHOT) and not head or managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_PLAYER_HEADSHOT_DOESNT_DO_DAMAGE) and head then
		damage = 0
		damage_percent = 0
	end

	if managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_ENEMIES_VULNERABLE_ONLY_TO_EXPLOSION) or managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_ENEMIES_VULNERABLE_ONLY_TO_MELEE) then
		damage = 0
		damage_percent = 0
	end

	if managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_PLAYER_HEADSHOT_AUTO_KILL) and head then
		damage = self._health
		damage_percent = 100
	end

	if attack_data.attacker_unit == managers.player:player_unit() and damage > 0 then
		if death_event_params.critical_hit then
			managers.hud:on_crit_confirmed()
		elseif death_event_params.headshot then
			managers.hud:on_headshot_confirmed()
		else
			local percent_dmg = damage / self._HEALTH_INIT

			if WeaponTweakData.HIT_INDICATOR_ABSOLUTE <= damage or WeaponTweakData.HIT_INDICATOR_PERCENT <= percent_dmg then
				managers.hud:on_hit_confirmed()
			end
		end

		if head then
			managers.player:on_headshot_dealt(self._health <= damage)
		end
	end

	if self._health <= damage then
		if head and math.random(100) < damage then
			self:_spawn_head_gadget({
				position = attack_data.col_ray.body:position(),
				rotation = attack_data.col_ray.body:rotation(),
				dir = attack_data.col_ray.ray
			})
		end

		attack_data.damage = self._health
		result = {
			type = "death",
			variant = attack_data.variant
		}

		self:die(attack_data)
		self:chk_killshot(attack_data.attacker_unit, WeaponTweakData.DAMAGE_TYPE_BULLET)

		if managers.buff_effect:is_effect_active(BuffEffectManager.EFFECT_PLAYER_KILL_REGENERATES_HEALTH) then
			local health_amount_regen = managers.buff_effect:get_effect_value(BuffEffectManager.EFFECT_PLAYER_KILL_REGENERATES_HEALTH) or 0

			if attack_data.attacker_unit == managers.player:local_player() then
				managers.player:local_player():character_damage():restore_health(health_amount_regen * managers.player:local_player():character_damage():get_max_health(), true)
			end
		end
	else
		attack_data.damage = damage
		local result_type = self:get_damage_type(damage_percent, WeaponTweakData.DAMAGE_TYPE_BULLET)
		result = {
			type = result_type,
			variant = attack_data.variant
		}

		self:_apply_damage_to_health(damage)
	end

	attack_data.result = result
	attack_data.pos = attack_data.col_ray.position
	local dismember_victim = false

	if result.type == "death" then
		if self:_dismember_condition(attack_data, false) then
			death_event_params.dismemberment_occured = true

			self:_dismember_body_part(attack_data)

			dismember_victim = true
		end

		local data = {
			unit_key = self._unit:key(),
			name = self._unit:base()._tweak_table,
			stats_name = self._unit:base()._stats_name,
			head_shot = head,
			weapon_unit = attack_data.weapon_unit,
			variant = attack_data.variant,
			dismembered = dismember_victim
		}

		if not attack_data.attacker_unit then
			Application:error("CopDamage:damage_bullet: missing attacker unit. ", inspect(attack_data))
		end

		if attack_data.attacker_unit and managers.groupai:state():all_criminals()[attack_data.attacker_unit:key()] then
			managers.statistics:killed_by_anyone(data)
		end

		if attack_data.attacker_unit == managers.player:player_unit() then
			death_event_params.enemy_distance = mvector3.distance(self._unit:position(), attack_data.attacker_unit:position())
			death_event_params.player_used_steelsight = managers.player:get_current_state():in_steelsight()
			death_event_params.weapon_used = attack_data.weapon_unit
			death_event_params.using_turret = managers.player:current_state() == "turret"

			if managers.player:upgrade_value("player", "warcry_health_regen_on_kill", false) == true and tweak_data.weapon[death_event_params.weapon_used:base()._name_id] and tweak_data.weapon[death_event_params.weapon_used:base()._name_id].category == WeaponTweakData.WEAPON_CATEGORY_SNP then
				local unit = managers.player:player_unit()
				local max_health = unit:character_damage():get_max_health()
				local current_health = unit:character_damage():get_real_health()
				local hp_pickup_amount = managers.player:upgrade_value("player", "warcry_health_regen_amount", false)

				if max_health < current_health + hp_pickup_amount then
					unit:character_damage():set_health(max_health)
				else
					unit:character_damage():set_health(current_health + hp_pickup_amount)
				end

				if managers.hud and managers.hud._sound_source and managers.hud._sound_source.post_event then
					managers.hud._sound_source:post_event(tweak_data.warcry.sharpshooter.health_boost_sound)
				end
			end

			self:_comment_death(attack_data.attacker_unit, self._unit:base()._tweak_table)
			self:_show_death_hint(self._unit:base()._tweak_table)

			local attacker_state = managers.player:current_state()
			data.attacker_state = attacker_state

			managers.statistics:killed(data)
			managers.system_event_listener:call_listeners(CoreSystemEventListenerManager.SystemEventListenerManager.PLAYER_KILLED_ENEMY, death_event_params)

			if attack_data.attacker_unit:character_damage():bleed_out() and not CopDamage.is_civilian(self._unit:base()._tweak_table) then
				local messiah_revive = false

				if managers.player:has_category_upgrade("player", "revive_from_bleed_out") and not data.weapon_unit:base().thrower_unit and attack_data.attacker_unit:character_damage():consume_messiah_charge() then
					messiah_revive = true
				end

				if messiah_revive then
					attack_data.attacker_unit:character_damage():revive(true)
				end
			end

			if not CopDamage.is_civilian(self._unit:base()._tweak_table) and managers.player:has_category_upgrade("temporary", "overkill_damage_multiplier") and not attack_data.weapon_unit:base().thrower_unit then
				local weapon_category = attack_data.weapon_unit:base():weapon_tweak_data().category

				if weapon_category == "shotgun" or weapon_category == "saw" then
					managers.player:activate_temporary_upgrade("temporary", "overkill_damage_multiplier")
				end
			end
		elseif attack_data.attacker_unit:in_slot(managers.slot:get_mask("criminals_no_deployables")) then
			self:_AI_comment_death(attack_data.attacker_unit, self._unit:base()._tweak_table)
		elseif attack_data.attacker_unit:base().sentry_gun and Network:is_server() then
			local server_info = attack_data.weapon_unit:base():server_information()

			if server_info and server_info.owner_peer_id ~= managers.network:session():local_peer():id() then
				local owner_peer = managers.network:session():peer(server_info.owner_peer_id)

				if owner_peer then
					owner_peer:send_queued_sync("sync_player_kill_statistic", data.name, data.head_shot and true or false, data.weapon_unit, data.variant, data.stats_name)
				end
			else
				data.attacker_state = managers.player:current_state()

				managers.statistics:killed(data)
			end
		end
	end

	local hit_offset_height = math.clamp(attack_data.col_ray.position.z - self._unit:movement():m_pos().z, 0, 300)
	local attacker = attack_data.attacker_unit

	if attacker:id() == -1 then
		attacker = self._unit
	end

	if alive(attack_data.weapon_unit) and attack_data.weapon_unit:base() and attack_data.weapon_unit:base().add_damage_result then
		attack_data.weapon_unit:base():add_damage_result(self._unit, attacker, result.type == "death", damage_percent)
	end

	self:_send_bullet_attack_result(attack_data, attacker, damage_percent, body_index, hit_offset_height)
	self:_on_damage_received(attack_data)

	return result
end